@@ -119,22 +119,29 @@ dy.d_ <- function(x, y, wrt,
   original.eval.points.max <- eval.points
   original.eval.points <- eval.points
 
-  h_s <- 1/log(length(x),c(2, 10))
-  h_s <- c(h_s, 10*h_s)
+  h_s <- abs(scale(LPM.VaR(seq(0, 1, .05), 1, x[,wrt]))+.5)
 
   if(NNS.dep(x[,wrt],y)$Dependence < .5) h_s <- 2*h_s
 
+  if(all(h_s>1)) h_s <- c(.2, .4, .5, .6, .8)
+  h_s <- h_s[h_s>0 & h_s<1]
+
+  h_s <- fivenum(h_s[h_s<1])
+
   for(h in h_s){
     index <- which(h == h_s)
     if(is.vector(eval.points) || dim(eval.points)[2] == 1){
       eval.points <- unlist(eval.points)
 
-      h_step <- abs(mean(diff(LPM.VaR(seq(.01, 1, h), 0, x[,wrt]))))
+      h_step <- mean(abs(diff(LPM.VaR(seq(0, 1, h), 1, x[,wrt]))))
+
+      if(h_step==0) h_step <- mean(abs(diff(LPM.VaR(seq(0, 1, .2)[-1], 1, x[,wrt]))))
+      if(h_step==0) h_step <- mean(x[,wrt])
 
       original.eval.points.min <- original.eval.points.min - h_step
       original.eval.points.max <- h_step + original.eval.points.max
 
-      deriv.points <- apply(x, 2, function(z) LPM.VaR(seq(0,1,.05), 0, z))
+      deriv.points <- apply(x, 2, function(z) LPM.VaR(seq(0,1,.05), 1, z))
 
       if(dim(deriv.points)[2]!=dim(x)[2]){
         deriv.points <- matrix(deriv.points, ncol = l, byrow = FALSE)
@@ -189,7 +196,10 @@ dy.d_ <- function(x, y, wrt,
       n <- dim(eval.points)[1]
       original.eval.points <- eval.points
 
-      h_step <- abs(mean(diff(LPM.VaR(seq(.01, 1, h), 0, x[,wrt]))))
+      h_step <- mean(abs(diff(LPM.VaR(seq(0, 1, h), 1, x[,wrt]))))
+
+      if(h_step==0) h_step <- mean(abs(diff(LPM.VaR(seq(0, 1, .2)[-1], 1, x[,wrt]))))
+      if(h_step==0) h_step <- abs(mean(x[,wrt]))
 
       original.eval.points.min[ , wrt] <- original.eval.points.min[ , wrt] - h_step
       original.eval.points.max[ , wrt] <- h_step + original.eval.points.max[ , wrt]
@@ -222,9 +232,14 @@ dy.d_ <- function(x, y, wrt,
       }
 
       if(!is.null(dim(eval.points))){
-        h_step_1 <- abs(mean(diff(LPM.VaR(seq(.01, 1, h), 0, x[ ,1]))))
+        h_step_1 <- mean(abs(diff(LPM.VaR(seq(0, 1, h), 1, x[ ,1]))))
+        if(h_step_1==0) h_step_1 <- mean(abs(diff(LPM.VaR(seq(0, 1, .2)[-1], 1, x[,1]))))
+        if(h_step_1==0) h_step_1 <- abs(mean(x[,1]))
 
-        h_step_2 <- abs(mean(diff(LPM.VaR(seq(.01, 1, h), 0, x[ ,2]))))
+
+        h_step_2 <- mean(abs(diff(LPM.VaR(seq(0, 1, h), 1, x[ ,2]))))
+        if(h_step_2==0) h_step_2 <- mean(abs(diff(LPM.VaR(seq(0, 1, .2)[-1], 1, x[,2]))))
+        if(h_step_2==0) h_step_2 <- abs(mean(x[,2]))
 
         mixed.deriv.points <- matrix(c(h_step_1 + eval.points[,1], h_step_2 + eval.points[,2],
                                        eval.points[,1] - h_step_1, h_step_2 + eval.points[,2],
